@@ -1,4 +1,4 @@
-  /* Pango
+/* Pango
  * pango-utils.c: Utilities for internal functions and modules
  *
  * Copyright (C) 2000 Red Hat Software
@@ -133,8 +133,8 @@ pango_version_string (void)
  **/
 const gchar*
 pango_version_check (int required_major,
-                     int required_minor,
-                     int required_micro)
+		     int required_minor,
+		     int required_micro)
 {
   gint pango_effective_micro = 100 * PANGO_VERSION_MINOR + PANGO_VERSION_MICRO;
   gint required_effective_micro = 100 * required_minor + required_micro;
@@ -151,9 +151,9 @@ pango_version_check (int required_major,
 /**
  * pango_trim_string:
  * @str: a string
- * 
+ *
  * Trims leading and trailing whitespace from a string.
- * 
+ *
  * Return value: A newly-allocated string that must be freed with g_free()
  **/
 char *
@@ -162,7 +162,7 @@ pango_trim_string (const char *str)
   int len;
 
   g_return_val_if_fail (str != NULL, NULL);
-  
+
   while (*str && g_ascii_isspace (*str))
     str++;
 
@@ -175,11 +175,11 @@ pango_trim_string (const char *str)
 
 /**
  * pango_split_file_list:
- * @str: a G_SEARCHPATH_SEPARATOR separated list of filenames
- * 
- * Splits a G_SEARCHPATH_SEPARATOR-separated list of files, stripping
- * white space and subsituting ~/ with $HOME/.
- * 
+ * @str: a %G_SEARCHPATH_SEPARATOR separated list of filenames
+ *
+ * Splits a %G_SEARCHPATH_SEPARATOR-separated list of files, stripping
+ * white space and substituting ~/ with $HOME/.
+ *
  * Return value: a list of strings to be freed with g_strfreev()
  **/
 char **
@@ -200,10 +200,10 @@ pango_split_file_list (const char *str)
 	{
 	  g_free(file);
 	  g_free (files[i]);
-	  
+
 	  for (j = i + 1; files[j]; j++)
 	    files[j - 1] = files[j];
-	  
+
 	  files[j - 1] = NULL;
 
 	  continue;
@@ -229,7 +229,7 @@ pango_split_file_list (const char *str)
 #endif
       g_free (files[i]);
       files[i] = file;
-	
+
       i++;
     }
 
@@ -240,7 +240,7 @@ pango_split_file_list (const char *str)
  * pango_read_line:
  * @stream: a stdio stream
  * @str: #GString buffer into which to write the result
- * 
+ *
  * Reads an entire line from a file into a buffer. Lines may
  * be delimited with '\n', '\r', '\n\r', or '\r\n'. The delimiter
  * is not written into the buffer. Text after a '#' character is treated as
@@ -248,10 +248,10 @@ pango_split_file_list (const char *str)
  * '\' proceeding a line delimiter combines adjacent lines. A '\' proceeding
  * any other character is ignored and written into the output buffer
  * unmodified.
- * 
+ *
  * Return value: 0 if the stream was already at an %EOF character, otherwise
  *               the number of lines read (this is useful for maintaining
- *               a line number counter which doesn't combine lines with '\') 
+ *               a line number counter which doesn't combine lines with '\')
  **/
 gint
 pango_read_line (FILE *stream, GString *str)
@@ -260,22 +260,22 @@ pango_read_line (FILE *stream, GString *str)
   gboolean comment = FALSE;
   int n_read = 0;
   int lines = 1;
-  
+
   flockfile (stream);
 
   g_string_truncate (str, 0);
-  
+
   while (1)
     {
       int c;
-      
+
       c = getc_unlocked (stream);
 
       if (c == EOF)
 	{
 	  if (quoted)
 	    g_string_append_c (str, '\\');
-	  
+
 	  goto done;
 	}
       else
@@ -284,7 +284,7 @@ pango_read_line (FILE *stream, GString *str)
       if (quoted)
 	{
 	  quoted = FALSE;
-	  
+
 	  switch (c)
 	    {
 	    case '#':
@@ -301,11 +301,11 @@ pango_read_line (FILE *stream, GString *str)
 		  ungetc (next_c, stream);
 
 		lines++;
-		
+
 		break;
 	      }
 	    default:
-	      g_string_append_c (str, '\\');	      
+	      g_string_append_c (str, '\\');
 	      g_string_append_c (str, c);
 	    }
 	}
@@ -348,9 +348,9 @@ pango_read_line (FILE *stream, GString *str)
 /**
  * pango_skip_space:
  * @pos: in/out string position
- * 
+ *
  * Skips 0 or more characters of white space.
- * 
+ *
  * Return value: %FALSE if skipping the white space leaves
  * the position at a '\0' character.
  **/
@@ -358,7 +358,7 @@ gboolean
 pango_skip_space (const char **pos)
 {
   const char *p = *pos;
-  
+
   while (g_ascii_isspace (*p))
     p++;
 
@@ -371,12 +371,12 @@ pango_skip_space (const char **pos)
  * pango_scan_word:
  * @pos: in/out string position
  * @out: a #GString into which to write the result
- * 
+ *
  * Scans a word into a #GString buffer. A word consists
  * of [A-Za-z_] followed by zero or more [A-Za-z_0-9]
  * Leading white space is skipped.
- * 
- * Return value: %FALSE if a parse error occured. 
+ *
+ * Return value: %FALSE if a parse error occurred.
  **/
 gboolean
 pango_scan_word (const char **pos, GString *out)
@@ -385,7 +385,7 @@ pango_scan_word (const char **pos, GString *out)
 
   while (g_ascii_isspace (*p))
     p++;
-  
+
   if (!((*p >= 'A' && *p <= 'Z') ||
 	(*p >= 'a' && *p <= 'z') ||
 	*p == '_'))
@@ -413,19 +413,19 @@ pango_scan_word (const char **pos, GString *out)
  * pango_scan_string:
  * @pos: in/out string position
  * @out: a #GString into which to write the result
- * 
+ *
  * Scans a string into a #GString buffer. The string may either
  * be a sequence of non-white-space characters, or a quoted
  * string with '"'. Instead a quoted string, '\"' represents
  * a literal quote. Leading white space outside of quotes is skipped.
- * 
- * Return value: %FALSE if a parse error occured.
+ *
+ * Return value: %FALSE if a parse error occurred.
  **/
 gboolean
 pango_scan_string (const char **pos, GString *out)
 {
   const char *p = *pos;
-  
+
   while (g_ascii_isspace (*p))
     p++;
 
@@ -443,7 +443,7 @@ pango_scan_string (const char **pos, GString *out)
 	  if (quoted)
 	    {
 	      int c = *p;
-	      
+
 	      switch (c)
 		{
 		case '\0':
@@ -454,8 +454,10 @@ pango_scan_string (const char **pos, GString *out)
 		case 't':
 		  c = '\t';
 		  break;
+		default:
+		  break;
 		}
-	      
+
 	      quoted = FALSE;
 	      g_string_append_c (out, c);
 	    }
@@ -501,11 +503,11 @@ pango_scan_string (const char **pos, GString *out)
  * pango_scan_int:
  * @pos: in/out string position
  * @out: an int into which to write the result
- * 
- * Scans an integer. 
+ *
+ * Scans an integer.
  * Leading white space is skipped.
- * 
- * Return value: %FALSE if a parse error occured.
+ *
+ * Return value: %FALSE if a parse error occurred.
  **/
 gboolean
 pango_scan_int (const char **pos, int *out)
@@ -521,7 +523,7 @@ pango_scan_int (const char **pos, int *out)
       errno = 0;
       return FALSE;
     }
-	
+
   *out = (int)temp;
   if ((long)(*out) != temp)
     {
@@ -543,7 +545,7 @@ read_config_file (const char *filename, gboolean enoent_error)
   gchar **groups;
   gsize groups_count = 0;
   guint group_index;
-  
+
   if (!g_key_file_load_from_file(key_file,filename, 0, &key_file_error))
     {
       if (key_file_error)
@@ -558,7 +560,7 @@ read_config_file (const char *filename, gboolean enoent_error)
       g_key_file_free(key_file);
       return;
     }
-  
+
   groups = g_key_file_get_groups (key_file, &groups_count);
   for (group_index = 0; group_index < groups_count; group_index++)
     {
@@ -566,9 +568,9 @@ read_config_file (const char *filename, gboolean enoent_error)
       const gchar *group = groups[group_index];
       GError *keys_error = NULL;
       gchar **keys;
-      
+
       keys = g_key_file_get_keys(key_file, group, &keys_count, &keys_error);
-      
+
       if (keys)
 	{
 	  guint key_index;
@@ -613,7 +615,7 @@ read_config (void)
       char *filename;
       const char *home;
       const char *envvar;
-      
+
       config_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 					   (GDestroyNotify)g_free,
 					   (GDestroyNotify)g_free);
@@ -640,11 +642,11 @@ read_config (void)
 /**
  * pango_config_key_get:
  * @key: Key to look up, in the form "SECTION/KEY".
- * 
+ *
  * Looks up a key in the Pango config database
  * (pseudo-win.ini style, read from $sysconfdir/pango/pangorc,
  *  ~/.pangorc, and getenv (PANGO_RC_FILE).)
- * 
+ *
  * Return value: the value, if found, otherwise %NULL. The value is a
  * newly-allocated string and must be freed with g_free().
  **/
@@ -652,7 +654,7 @@ char *
 pango_config_key_get (const char *key)
 {
   g_return_val_if_fail (key != NULL, NULL);
-  
+
   read_config ();
 
   return g_strdup (g_hash_table_lookup (config_hash, key));
@@ -674,7 +676,7 @@ G_WIN32_DLLMAIN_FOR_DLL_NAME(static, dll_name)
  * the DLL's location, or stored in the Registry).
  *
  * Return value: the Pango sysconf directory. The returned string should
- * not be freed. 
+ * not be freed.
  */
 G_CONST_RETURN char *
 pango_get_sysconf_subdirectory (void)
@@ -702,7 +704,7 @@ pango_get_sysconf_subdirectory (void)
  * not be freed.
  *
  * Return value: the Pango lib directory. The returned string should
- * not be freed. 
+ * not be freed.
  */
 G_CONST_RETURN char *
 pango_get_lib_subdirectory (void)
@@ -720,18 +722,94 @@ pango_get_lib_subdirectory (void)
 #endif
 }
 
+
+/**
+ * pango_parse_enum:
+ * @type: enum type to parse, eg. %PANGO_TYPE_ELLIPSIZE_MODE.
+ * @str: string to parse.  May be %NULL.
+ * @value: integer to store the result in, or %NULL.
+ * @warn: if %TRUE, issue a g_warning() on bad input.
+ * @possible_values: place to store list of possible values on failure, or %NULL.
+ *
+ * Parses an enum type and stored the result in @value.
+ *
+ * If @str does not match the nick name of any of the possible values for the
+ * enum, %FALSE is returned, a warning is issued if @warn is %TRUE, and a
+ * string representing the list of possible values is stored in
+ * @possible_values.  The list is slash-separated, eg.
+ * "none/start/middle/end".  If failed and @possible_values is not %NULL,
+ * returned string should be freed using g_free().
+ *
+ * Return value: %TRUE if @str was successfully parsed.
+ *
+ * Since: 1.16
+ **/
+gboolean
+pango_parse_enum (GType       type,
+		  const char *str,
+		  int        *value,
+		  gboolean    warn,
+		  char      **possible_values)
+{
+  GEnumClass *class = NULL;
+  gboolean ret = TRUE;
+  GEnumValue *v = NULL;
+
+  class = g_type_class_ref (type);
+
+  if (G_LIKELY (str))
+    v = g_enum_get_value_by_nick (class, str);
+
+  if (v)
+    {
+      if (G_LIKELY (value))
+	*value = v->value;
+    }
+  else
+    {
+      ret = FALSE;
+      if (warn || possible_values)
+	{
+	  int i;
+	  GString *s = g_string_new (NULL);
+
+	  for (i = 0, v = g_enum_get_value (class, i); v;
+	       i++  , v = g_enum_get_value (class, i))
+	    {
+	      if (i)
+		g_string_append_c (s, '/');
+	      g_string_append (s, v->value_nick);
+	    }
+
+	  if (warn)
+	    g_warning ("%s must be one of %s",
+		       G_ENUM_CLASS_TYPE_NAME(class),
+		       s->str);
+
+	  if (possible_values)
+	    *possible_values = s->str;
+
+	  g_string_free (s, possible_values ? FALSE : TRUE);
+	}
+    }
+
+  g_type_class_unref (class);
+
+  return ret;
+}
+
 /**
  * pango_parse_style:
  * @str: a string to parse.
  * @style: a #PangoStyle to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
- * 
+ *
  * Parses a font style. The allowed values are "normal",
  * "italic" and "oblique", case variations being
  * ignored.
- * 
+ *
  * Return value: %TRUE if @str was successfully parsed.
- **/ 
+ **/
 gboolean
 pango_parse_style (const char *str,
 		   PangoStyle *style,
@@ -769,7 +847,7 @@ pango_parse_style (const char *str,
     }
   if (warn)
     g_warning ("style must be normal, italic, or oblique");
-  
+
   return FALSE;
 }
 
@@ -778,13 +856,13 @@ pango_parse_style (const char *str,
  * @str: a string to parse.
  * @variant: a #PangoVariant to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
- * 
+ *
  * Parses a font variant. The allowed values are "normal"
  * and "smallcaps" or "small_caps", case variations being
  * ignored.
- * 
+ *
  * Return value: %TRUE if @str was successfully parsed.
- **/ 
+ **/
 gboolean
 pango_parse_variant (const char   *str,
 		     PangoVariant *variant,
@@ -813,7 +891,7 @@ pango_parse_variant (const char   *str,
 	}
       break;
     }
-  
+
   if (warn)
     g_warning ("variant must be normal or small_caps");
   return FALSE;
@@ -824,13 +902,13 @@ pango_parse_variant (const char   *str,
  * @str: a string to parse.
  * @weight: a #PangoWeight to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
- * 
+ *
  * Parses a font weight. The allowed values are "heavy",
- * "ultrabold", "bold", "normal", "light", "ultraleight" 
+ * "ultrabold", "bold", "normal", "light", "ultraleight"
  * and integers. Case variations are ignored.
- * 
+ *
  * Return value: %TRUE if @str was successfully parsed.
- **/ 
+ **/
 gboolean
 pango_parse_weight (const char  *str,
 		    PangoWeight *weight,
@@ -909,7 +987,7 @@ pango_parse_weight (const char  *str,
 	return TRUE;
       }
     }
-  
+
   if (warn)
     g_warning ("weight must be ultralight, light, normal, bold, ultrabold, heavy, or an integer");
   return FALSE;
@@ -920,15 +998,15 @@ pango_parse_weight (const char  *str,
  * @str: a string to parse.
  * @stretch: a #PangoStretch to store the result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
- * 
- * Parses a font stretch. The allowed values are 
- * "ultra_condensed", "extra_condensed", "condensed", 
- * "semi_condensed", "normal", "semi_expanded", "expanded", 
- * "extra_expanded" and "ultra_expanded". Case variations are 
+ *
+ * Parses a font stretch. The allowed values are
+ * "ultra_condensed", "extra_condensed", "condensed",
+ * "semi_condensed", "normal", "semi_expanded", "expanded",
+ * "extra_expanded" and "ultra_expanded". Case variations are
  * ignored and the '_' characters may be omitted.
- * 
+ *
  * Return value: %TRUE if @str was successfully parsed.
- **/ 
+ **/
 gboolean
 pango_parse_stretch (const char   *str,
 		     PangoStretch *stretch,
@@ -938,7 +1016,7 @@ pango_parse_stretch (const char   *str,
     return FALSE;
 
   switch (str[0])
-    { 
+    {
     case 'c':
     case 'C':
       if (g_ascii_strcasecmp (str, "condensed") == 0)
@@ -1012,343 +1090,11 @@ pango_parse_stretch (const char   *str,
   return FALSE;
 }
 
-static const char canon_map[256] = {
-   0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
-   0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,   0,   0,   0, 
-   0,   0,   0,   0,   0,   0,   0,   0,    0,   0,   0,   0,   0,  '-',  0,   0, 
-  '0', '1', '2', '3', '4', '5', '6', '7',  '8', '9',  0,   0,   0,   0,   0,   0, 
-   0,  'a', 'b', 'c', 'd', 'e', 'f', 'g',  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
-  'p', 'q', 'r', 's', 't', 'u', 'v', 'w',  'x', 'y', 'z',  0,   0,   0,   0,  '-',
-   0,  'a', 'b', 'c', 'd', 'e', 'f', 'g',  'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
-  'p', 'q', 'r', 's', 't', 'u', 'v', 'w',  'x', 'y', 'z',  0,   0,   0,   0,   0
-};
-   
-static gboolean
-lang_equal (gconstpointer v1,
-	    gconstpointer v2)
-{
-  const guchar *p1 = v1;
-  const guchar *p2 = v2;
-
-  while (canon_map[*p1] && canon_map[*p1] == canon_map[*p2])
-    {
-      p1++, p2++;
-    }
-
-  return (canon_map[*p1] == canon_map[*p2]);
-}
-
-static guint
-lang_hash (gconstpointer key)
-{
-  const guchar *p = key;
-  guint h = 0;
-  while (canon_map[*p])
-    {
-      h = (h << 5) - h + canon_map[*p];
-      p++;
-    }
-
-  return h;
-}
-
-static PangoLanguage *
-pango_language_copy (PangoLanguage *language)
-{
-  return language; /* language tags are const */
-}
-
-static void
-pango_language_free (PangoLanguage *language)
-{
-  return; /* nothing */
-}
-
-GType
-pango_language_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static (I_("PangoLanguage"),
-                                             (GBoxedCopyFunc)pango_language_copy,
-                                             (GBoxedFreeFunc)pango_language_free);
-  return our_type;
-}
-
-/**
- * _pango_get_lc_ctype:
- *
- * Return the Unix-style locale string for the language currently in
- * effect. On Unix systems, this is the return value from
- * <literal>setlocale(LC_CTYPE, NULL)</literal>, and the user can
- * affect this through the environment variables LC_ALL, LC_CTYPE or
- * LANG (checked in that order). The locale strings typically is in
- * the form lang_COUNTRY, where lang is an ISO-639 language code, and
- * COUNTRY is an ISO-3166 country code. For instance, sv_FI for
- * Swedish as written in Finland or pt_BR for Portuguese as written in
- * Brazil.
- * 
- * On Windows, the C library doesn't use any such environment
- * variables, and setting them won't affect the behaviour of functions
- * like ctime(). The user sets the locale through the Regional Options 
- * in the Control Panel. The C library (in the setlocale() function) 
- * does not use country and language codes, but country and language 
- * names spelled out in English. 
- * However, this function does check the above environment
- * variables, and does return a Unix-style locale string based on
- * either said environment variables or the thread's current locale.
- *
- * Return value: a dynamically allocated string, free with g_free().
- */
-static gchar *
-_pango_get_lc_ctype (void)
-{
-#ifdef G_OS_WIN32
-  /* Somebody might try to set the locale for this process using the
-   * LANG or LC_ environment variables. The Microsoft C library
-   * doesn't know anything about them. You set the locale in the
-   * Control Panel. Setting these env vars won't have any affect on
-   * locale-dependent C library functions like ctime(). But just for
-   * kicks, do obey LC_ALL, LC_CTYPE and LANG in Pango. (This also makes
-   * it easier to test GTK and Pango in various default languages, you
-   * don't have to clickety-click in the Control Panel, you can simply
-   * start the program with LC_ALL=something on the command line.)
-   */
-
-  gchar *p;
-
-  p = getenv ("LC_ALL");
-  if (p != NULL)
-    return g_strdup (p);
-
-  p = getenv ("LC_CTYPE");
-  if (p != NULL)
-    return g_strdup (p);
-
-  p = getenv ("LANG");
-  if (p != NULL)
-    return g_strdup (p);
-
-  return g_win32_getlocale ();
-#else
-  return g_strdup (setlocale (LC_CTYPE, NULL));
-#endif
-}
-
-/**
- * pango_language_get_default:
- *
- * Returns the #PangoLanguage for the current locale of the process.
- * Note that this can change over the life of an application.
- *
- * Your application should call <literal>setlocale (LC_ALL, "");</literal>
- * for the user settings to take effect.  Gtk+ does this in its initialization
- * functions.  See <literal>man setlocale</literal> for details.
- * 
- * Return value: the default language as a #PangoLanguage, must not be
- *               freed.
- *
- * Since: 1.16
- **/
-PangoLanguage *
-pango_language_get_default (void)
-{
-  gchar *lang;
-  PangoLanguage *result;  
-  
-  lang = _pango_get_lc_ctype ();
- 
-  result = pango_language_from_string (lang);
-  g_free (lang);
-  
-  return result;
-}
-
-/**
- * pango_language_from_string:
- * @language: a string representing a language tag
- * 
- * Take a RFC-3066 format language tag as a string and convert it to a
- * #PangoLanguage pointer that can be efficiently copied (copy the
- * pointer) and compared with other language tags (compare the
- * pointer.)
- *
- * This function first canonicalizes the string by converting it to
- * lowercase, mapping '_' to '-', and stripping all characters other
- * than letters and '-'.
- *
- * Use pango_language_get_default() if you want to get the #PangoLanguage for
- * the current locale of the process.
- * 
- * Return value: an opaque pointer to a #PangoLanguage structure.
- *               this will be valid forever after.
- **/
-PangoLanguage *
-pango_language_from_string (const char *language)
-{
-  static GHashTable *hash = NULL;
-  char *result;
-  int len;
-  char *p;
-
-  if (G_UNLIKELY (!hash))
-    hash = g_hash_table_new (lang_hash, lang_equal);
-  else
-    {
-      result = g_hash_table_lookup (hash, language);
-      if (result)
-	return (PangoLanguage *)result;
-    }
-
-  len = strlen (language);
-  result = g_malloc (len + 1);
-
-  p = result;
-  while ((*(p++) = canon_map[*(guchar *)language++]))
-    ;
-
-  g_hash_table_insert (hash, result, result);
-
-  return (PangoLanguage *)result;
-}
-
-/**
- * pango_language_matches:
- * @language: a language tag (see pango_language_from_string()),
- *            %NULL is allowed and matches nothing but '*'
- * @range_list: a list of language ranges, separated by ';', ':',
- *   ',', or space characters.
- *   Each element must either be '*', or a RFC 3066 language range
- *   canonicalized as by pango_language_from_string()
- * 
- * Checks if a language tag matches one of the elements in a list of
- * language ranges. A language tag is considered to match a range
- * in the list if the range is '*', the range is exactly the tag,
- * or the range is a prefix of the tag, and the character after it
- * in the tag is '-'.
- *
- * Return value: %TRUE if a match was found.
- **/
-gboolean
-pango_language_matches (PangoLanguage *language,
-			const char    *range_list)
-{
-  const char *lang_str = pango_language_to_string (language);
-  const char *p = range_list;
-  gboolean done = FALSE;
-
-  while (!done)
-    {
-      const char *end = strpbrk (p, ";:, \t");
-      if (!end)
-	{
-	  end = p + strlen (p);
-	  done = TRUE;
-	}
-
-      if (strncmp (p, "*", 1) == 0 ||
-	  (lang_str && strncmp (lang_str, p, end - p) == 0 &&
-	   (lang_str[end - p] == '\0' || lang_str[end - p] == '-')))
-	return TRUE;
-
-      if (!done)
-	p = end + 1;
-    }
-
-  return FALSE;
-}
-
-typedef struct {
-  const char lang[4];
-  const char *str;
-} LangInfo;
-
-static int
-lang_info_compare (const void *key, const void *val)
-{
-  const LangInfo *lang_info = val;
-
-  return strncmp (key, lang_info->lang, 2);
-}
-
-/* The following array is supposed to contain enough text to tickle all necessary fonts for each
- * of the languages in the following. Yes, it's pretty lame. Not all of the languages
- * in the following have sufficient text to excercise all the accents for the language, and
- * there are obviously many more languages to include as well.
- */
-static const LangInfo lang_texts[] = {
-  { "ar", "Arabic  \330\247\331\204\330\263\331\204\330\247\331\205 \330\271\331\204\331\212\331\203\331\205" },
-  { "cs", "Czech (\304\215esky)  Dobr\303\275 den" },
-  { "da", "Danish (Dansk)  Hej, Goddag" },
-  { "el", "Greek (\316\225\316\273\316\273\316\267\316\275\316\271\316\272\316\254) \316\223\316\265\316\271\316\254 \317\203\316\261\317\202" },
-  { "en", "English Hello" },
-  { "eo", "Esperanto Saluton" },
-  { "es", "Spanish (Espa\303\261ol) \302\241Hola!" },
-  { "et", "Estonian  Tere, Tervist" },
-  { "fi", "Finnish (Suomi)  Hei, Hyv\303\244\303\244 p\303\244iv\303\244\303\244" },
-  { "fr", "French (Fran\303\247ais)" },
-  { "de", "German Gr\303\274\303\237 Gott" },
-  { "he", "Hebrew   \327\251\327\234\327\225\327\235" },
-  { "it", "Italiano  Ciao, Buon giorno" },
-  { "ja", "Japanese (\346\227\245\346\234\254\350\252\236) \343\201\223\343\202\223\343\201\253\343\201\241\343\201\257, \357\275\272\357\276\235\357\276\206\357\276\201\357\276\212" },
-  { "ko", "Korean (\355\225\234\352\270\200)   \354\225\210\353\205\225\355\225\230\354\204\270\354\232\224, \354\225\210\353\205\225\355\225\230\354\213\255\353\213\210\352\271\214" },
-  { "mt", "Maltese   \304\212aw, Sa\304\247\304\247a" },
-  { "nl", "Nederlands, Vlaams Hallo, Dag" },
-  { "no", "Norwegian (Norsk) Hei, God dag" },
-  { "pl", "Polish   Dzie\305\204 dobry, Hej" },
-  { "ru", "Russian (\320\240\321\203\321\201\321\201\320\272\320\270\320\271)" },
-  { "sk", "Slovak   Dobr\303\275 de\305\210" },
-  { "sv", "Swedish (Svenska) Hej p\303\245 dej, Goddag" },
-  { "tr", "Turkish (T\303\274rk\303\247e) Merhaba" },
-  { "zh", "Chinese (\344\270\255\346\226\207,\346\231\256\351\200\232\350\257\235,\346\261\211\350\257\255)" }
-};
-
-/**
- * pango_language_get_sample_string:
- * @language: a #PangoLanguage
- * 
- * Get a string that is representative of the characters needed to
- * render a particular language. This function is a bad hack for
- * internal use by renderers and Pango.
- * 
- * Return value: the sample string. This value is owned by Pango
- *   and must not be freed.
- **/
-G_CONST_RETURN char *
-pango_language_get_sample_string (PangoLanguage *language)
-{
-  const char *result;
-  
-  if (language)
-    {
-      const char *lang_str = pango_language_to_string (language);
-      
-      LangInfo *lang_info = bsearch (lang_str, lang_texts,
-				     G_N_ELEMENTS (lang_texts), sizeof (LangInfo),
-				     lang_info_compare);
-
-      if (lang_info)
-	result = lang_info->str;
-      else
-	result = "French (Fran\303\247ais)";     /* Assume iso-8859-1 */
-    }
-  else
-    {
-      /* Complete junk
-       */
-
-      result = "\330\247\331\204\330\263\331\204\330\247\331\205 \330\271\331\204\331\212\331\203\331\205 \304\215esky \316\225\316\273\316\273\316\267\316\275\316\271\316\272\316\254 Fran\303\247ais \346\227\245\346\234\254\350\252\236 \355\225\234\352\270\200 \320\240\321\203\321\201\321\201\320\272\320\270\320\271 \344\270\255\346\226\207,\346\231\256\351\200\232\350\257\235,\346\261\211\350\257\255 T\303\274rk\303\247e";
-    }
-
-  return result;
-}
-
 /**
  * pango_log2vis_get_embedding_levels:
  * @text:      the text to itemize.
  * @length:    the number of bytes (not characters) to process, or -1
- *             if @text is nul-terminated and the legnth should be calculated.
+ *             if @text is nul-terminated and the length should be calculated.
  * @pbase_dir: input base direction, and output resolved direction.
  *
  * This will return the bidirectional embedding levels of the input paragraph
@@ -1453,15 +1199,15 @@ pango_unichar_direction (gunichar ch)
  *
  * Use g_unichar_get_mirror_char() instead; the docs for that function
  * provide full details.
- * 
+ *
  * Return value: %TRUE if @ch has a mirrored character and @mirrored_ch is
  * filled in, %FALSE otherwise
  **/
-gboolean 
+gboolean
 pango_get_mirror_char (gunichar        ch,
 		       gunichar       *mirrored_ch)
 {
-  return g_unichar_get_mirror_char (ch, mirrored_ch); 
+  return g_unichar_get_mirror_char (ch, mirrored_ch);
 }
 
 
@@ -1490,7 +1236,7 @@ alias_free (struct PangoAlias *alias)
     g_free (alias->families[i]);
 
   g_free (alias->families);
-  
+
   g_slice_free (struct PangoAlias, alias);
 }
 
@@ -1498,7 +1244,7 @@ static void
 read_alias_file (const char *filename)
 {
   FILE *file;
-    
+
   GString *line_buffer;
   GString *tmp_buffer1;
   GString *tmp_buffer2;
@@ -1527,14 +1273,14 @@ read_alias_file (const char *filename)
       pos = line_buffer->str;
       if (!pango_skip_space (&pos))
 	continue;
-      
+
       if (!pango_scan_word (&pos, tmp_buffer1) ||
 	  !pango_skip_space (&pos))
 	{
 	  errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
 	  goto error;
 	}
-      
+
       if (*pos == '+')
 	{
 	  append = TRUE;
@@ -1546,7 +1292,7 @@ read_alias_file (const char *filename)
 	  errstring = g_strdup ("Line is not of the form KEY=VALUE or KEY+=VALUE");
 	  goto error;
 	}
-      
+
       if (!pango_scan_string (&pos, tmp_buffer2))
 	{
 	  errstring = g_strdup ("Error parsing value string");
@@ -1559,27 +1305,27 @@ read_alias_file (const char *filename)
 	}
 
       alias_key.alias = g_ascii_strdown (tmp_buffer1->str, -1);
-      
+
       /* Remove any existing values */
       alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
-      
+
       if (!alias)
 	{
 	  alias = g_slice_new0 (struct PangoAlias);
 	  alias->alias = alias_key.alias;
-	  
+
 	  g_hash_table_insert (pango_aliases_ht,
 			       alias, alias);
 	}
       else
 	g_free (alias_key.alias);
-	  
+
       new_families = g_strsplit (tmp_buffer2->str, ",", -1);
-      
+
       n_new = 0;
       while (new_families[n_new])
 	n_new++;
-      
+
       if (alias->families && append)
 	{
 	  alias->families = g_realloc (alias->families,
@@ -1594,7 +1340,7 @@ read_alias_file (const char *filename)
 	  for (i = 0; i < alias->n_families; i++)
 	    g_free (alias->families[i]);
 	  g_free (alias->families);
-	  
+
 	  alias->families = new_families;
 	  alias->n_families = n_new;
 	}
@@ -1602,7 +1348,7 @@ read_alias_file (const char *filename)
 
   if (ferror (file))
     errstring = g_strdup (g_strerror(errno));
-  
+
  error:
 
   if (errstring)
@@ -1610,7 +1356,7 @@ read_alias_file (const char *filename)
       g_warning ("error reading alias file: %s:%d: %s\n", filename, line, errstring);
       g_free (errstring);
     }
-      
+
   g_string_free (line_buffer, TRUE);
   g_string_free (tmp_buffer1, TRUE);
   g_string_free (tmp_buffer2, TRUE);
@@ -1635,7 +1381,7 @@ pango_load_aliases (void)
 			  NULL);
   read_alias_file (filename);
   g_free (filename);
-  
+
   home = g_get_home_dir ();
   if (home && *home)
     {
@@ -1666,7 +1412,7 @@ pango_lookup_aliases (const char   *fontname,
 {
   struct PangoAlias alias_key;
   struct PangoAlias *alias;
-  
+
   if (pango_aliases_ht == NULL)
     pango_load_aliases ();
 
@@ -1691,7 +1437,7 @@ pango_lookup_aliases (const char   *fontname,
  * @text:   the text to process
  * @length: length of @text in bytes (may be -1 if @text is nul-terminated)
  *
- * Searches a string the first character that has a strong 
+ * Searches a string the first character that has a strong
  * direction, according to the Unicode bidirectional algorithm.
  *
  * Return value: The direction corresponding to the first strong character.
@@ -1728,8 +1474,11 @@ pango_find_base_dir (const gchar *text,
  * pango_is_zero_width:
  * @ch: a Unicode character
  *
- * Checks @ch to see if it is a zero-width character that should not be
- * normally rendered on the screen.
+ * Checks @ch to see if it is a character that should not be
+ * normally rendered on the screen.  This includes all Unicode characters
+ * with "ZERO WIDTH" in their name, as well as <firstterm>bidi</firstterm> formatting characters, and
+ * a few other ones.  This is totally different from g_unichar_iszerowidth()
+ * and is at best misnamed.
  *
  * Return value: %TRUE if @ch is a zero-width character, %FALSE otherwise
  *
@@ -1793,7 +1542,7 @@ pango_quantize_line_geometry (int *thickness,
   int thickness_pixels = (*thickness + PANGO_SCALE / 2) / PANGO_SCALE;
   if (thickness_pixels == 0)
     thickness_pixels = 1;
-  
+
   if (thickness_pixels & 1)
     {
       int new_center = ((*position - *thickness / 2) & ~(PANGO_SCALE - 1)) + PANGO_SCALE / 2;
@@ -1809,30 +1558,86 @@ pango_quantize_line_geometry (int *thickness,
 }
 
 /**
- * pango_gravity_to_rotation:
- * @gravity: gravity to query
+ * pango_units_from_double:
+ * @d: double floating-point value
  *
- * Converts a #PangoGravity value to its rotation value.
+ * Converts a floating-point number to Pango units: multiplies
+ * it by %PANGO_SCALE and rounds to nearest integer.
  *
- * Return value: the rotation value corresponding to @gravity,
- * or zero if @gravity is %PANGO_GRAVITY_AUTO
+ * Return value: the value in Pango units.
+ *
+ * Since: 1.16
+ */
+int
+pango_units_from_double (double d)
+{
+  return (int)floor (d * PANGO_SCALE + 0.5);
+}
+
+/**
+ * pango_units_to_double:
+ * @i: value in Pango units
+ *
+ * Converts a number in Pango units to floating-point: divides
+ * it by %PANGO_SCALE.
+ *
+ * Return value: the double value.
  *
  * Since: 1.16
  */
 double
-pango_gravity_to_rotation (PangoGravity gravity)
+pango_units_to_double (int i)
 {
-  double rotation;
-
-  switch (gravity)
-    {
-      default:
-      case PANGO_GRAVITY_SOUTH:	rotation =  0;		break;
-      case PANGO_GRAVITY_NORTH:	rotation =  G_PI;	break;
-      case PANGO_GRAVITY_EAST:	rotation = -G_PI_2;	break;
-      case PANGO_GRAVITY_WEST:	rotation = +G_PI_2;	break;
-    }
-
-  return rotation;
+  return (double)i / PANGO_SCALE;
 }
 
+/**
+ * pango_extents_to_pixels:
+ * @ink_rect: ink rectangle to convert, or %NULL.
+ * @logical_rect: logical rectangle to convert, or %NULL.
+ *
+ * Converts extents from Pango units to device units, dividing by the
+ * %PANGO_SCALE factor and performing rounding.
+ *
+ * The ink rectangle is converted by flooring the x/y coordinates and extending
+ * width/height, such that the final rectangle completely includes the original
+ * rectangle.
+ *
+ * The logical rectangle is converted by rounding the coordinates
+ * of the rectangle to the nearest device unit.
+ *
+ * Note that in certain situations you may want pass a logical extents
+ * rectangle to this function as @ink_rect.  The rule is: if you want the
+ * resulting device-space rectangle to completely contain the original
+ * rectangle, pass it in as @ink_rect.
+ *
+ * Since: 1.16
+ **/
+void
+pango_extents_to_pixels (PangoRectangle *ink_rect,
+			 PangoRectangle *logical_rect)
+{
+  if (ink_rect)
+    {
+      int orig_x = ink_rect->x;
+      int orig_y = ink_rect->y;
+
+      ink_rect->x = PANGO_PIXELS_FLOOR (ink_rect->x);
+      ink_rect->y = PANGO_PIXELS_FLOOR (ink_rect->y);
+
+      ink_rect->width  = PANGO_PIXELS_CEIL (orig_x + ink_rect->width ) - ink_rect->x;
+      ink_rect->height = PANGO_PIXELS_CEIL (orig_y + ink_rect->height) - ink_rect->y;
+    }
+
+  if (logical_rect)
+    {
+      int orig_x = logical_rect->x;
+      int orig_y = logical_rect->y;
+
+      logical_rect->x = PANGO_PIXELS (logical_rect->x);
+      logical_rect->y = PANGO_PIXELS (logical_rect->y);
+
+      logical_rect->width  = PANGO_PIXELS (orig_x + logical_rect->width ) - logical_rect->x;
+      logical_rect->height = PANGO_PIXELS (orig_y + logical_rect->height) - logical_rect->y;
+    }
+}
